from pydantic import BaseModel
from pydantic import Field


class PageMeta(BaseModel):
    """Page meta data.

    Represents the meta data of a page of results.
    offset, page, pages may be None in various pagination combinations
    """

    total: int
    limit: int
    offset: int | None
    page: int | None
    pages: int | None


class MovieModel(BaseModel):
    """Single movie model."""

    id: str = Field(alias="_id")
    name: str
    runtime_in_minutes: int = Field(alias="runtimeInMinutes")
    budget_in_millions: int = Field(alias="budgetInMillions")
    box_office_revenue_in_millions: int = Field(alias="boxOfficeRevenueInMillions")
    academy_award_nominations: int = Field(alias="academyAwardNominations")
    academy_award_wins: int = Field(alias="academyAwardWins")
    rotten_tomatoes_score: int = Field(alias="rottenTomatoesScore")


class MoviesModel(PageMeta):
    """Multiple movies model.

    Contains a list (page) of movies and page meta data.
    """

    docs: list[MovieModel]


class QuoteModel(BaseModel):
    """Single movie model."""

    id: str = Field(alias="_id")
    dialog: str
    movie: str
    character: str


class QuotesModel(PageMeta):
    """Multiple quotes model.

    Contains a list (page) of movies and page meta data.
    """

    docs: list[QuoteModel]



from pydantic import BaseSettings
from pydantic.env_settings import SettingsSourceCallable


class Settings(BaseSettings):
    """Settings for the LOTR API client."""

    BEARER_TOKEN: str

    class Config(BaseSettings.Config):
        """Settings configuration.

        Main purpose is to change the order of the settings sources to (in order): Init value,
        Environment variable, Secret File (secret) (not tested)

        Please see base class for more information
        Docs - https://docs.pydantic.dev/latest/usage/settings/#customise-settings-sources
        """

        @classmethod
        def customise_sources(  # noqa: D102
            cls,
            init_settings: SettingsSourceCallable,
            env_settings: SettingsSourceCallable,
            file_secret_settings: SettingsSourceCallable,
        ) -> tuple[SettingsSourceCallable, ...]:
            return env_settings, init_settings, file_secret_settings



import time
from math import ceil

import httpx


def raise_for_status(res: httpx.Response) -> None:
    """Exception helper `for httpx.Response`.

    This helper raises an exception if the response status code is 4xx or 5xx
    Additionally sets custom error message for 429 status code with time to retry in (seconds)
    More elegant way would be subclassing the `httpx.Client()` and adding custom message,
    however for the sake of time we use this helper

    Args:
        res : `httpx.Response` object

    Raises:
        HTTPStatusError : If status code is 4xx or 5xx
    """
    if res.status_code == 429:  # noqa: WPS432
        retry_in = int(res.headers["X-RateLimit-Reset"]) - ceil(time.time())
        raise httpx.HTTPStatusError(
            f"Rate limit exceeded, retry after {retry_in}s",
            request=res.request,
            response=res,
        )
    res.raise_for_status()
